// Copyright © 2016-2017 Graham Sanderson, all rights reserved.
#include "lith_upgrades_common.h"

// Static Functions ----------------------------------------------------------|

//
// HUD_Weapons
//
static void HUD_Weapons(struct player *p)
{
   DrawSpritePlain(":HUD_C:Bar", hid_weaponbg, 320.2, 220.2, TS);

   static int const ncolor[] = {
      CR_DARKRED,
      CR_DARKGREEN,
      CR_BLUE
   };

   Lith_HUD_DrawWeaponSlots(p, ncolor, countof(ncolor), 'g', 323, 219);
}

//
// HUD_Ammo
//
static void HUD_Ammo(struct player *p)
{
   invweapon_t  const *wep  = p->weapon.cur;
   weaponinfo_t const *info = wep->info;

   __str typegfx = null;

   if(wep->ammotype & AT_NMag)
   {
      int max = ACS_GetMaxInventory(0, wep->magclass);
      int cur = ACS_CheckInventory(wep->magclass);

      typegfx = ":HUD_C:MAG";
      HudMessageF("LHUDFONT", "%i/%i", max - cur, max);
      HudMessageParams(0, hid_ammo1, CR_DARKRED, 242.1, 229.0, TS);
   }

   if(wep->ammotype & AT_Ammo && !(wep->info->flags & wf_magic))
   {
      int x = 0;

      if(wep->ammotype & AT_NMag) {
         DrawSpritePlain(":HUD_C:Back", hid_ammobg2, 240.2, 238.2, TS);
         x = -58;
      }

      typegfx = ":HUD_C:AMMO";
      HudMessageF("LHUDFONT", "%i", ACS_CheckInventory(wep->ammoclass));
      HudMessageParams(0, hid_ammo2, CR_DARKRED, x+242.1, 229.0, TS);
   }

   if(typegfx) {
      DrawSpritePlain(":HUD_C:SplitFront", hid_ammobg1, 320.2, 238.2, TS);
      DrawSpritePlain(typegfx, hid_ammotype, 316.2, 235.2, TS);
   }

   if(p->weapontype == weapon_c_smg)
   {
      DrawSpritePlain(":HUD_C:BarSmall", hid_rifleheatbg, 320.2, 205.2, TS);
      ACS_SetHudClipRect(320-63, 205-9, (ACS_CheckInventory("Lith_SMGHeat")/500.k) * 63, 9);
      DrawSpritePlain(":HUD_C:HeatBar", hid_rifleheat, 320.2, 205.2, TS);
      ACS_SetHudClipRect(0, 0, 0, 0);
   }
}

//
// HUD_Health
//
static void HUD_Health(struct player *p)
{
   DrawSpritePlain(":HUD_C:SplitBack", hid_healthbg, 0.1, 239.2, TS);

   if(p->dead) HudMessageF("LHUDFONT", "---");
   else        HudMessageF("LHUDFONT", "%i", p->health);
   HudMessageParams(0, hid_health, CR_DARKRED, 21.1, 229.0, TS);

   DrawSpritePlain(":HUD_C:VIT", hid_healthtxt, 2.1, 237.2, TS);
}

//
// HUD_Score
//
static void HUD_Score(struct player *p)
{
   HudMessageF("CNFONT", "\Ca%S\Cnscr", Lith_ScoreSep(p->score));
   HudMessageParams(HUDMSG_PLAIN, hid_score, CR_WHITE, 160.0, 3.1, 0.1);
}

//
// HUD_KeyInd
//
static void HUD_KeyInd(struct player *p)
{
   if(p->keys.rs) DrawSpriteAlpha(":HUD:H_KS1", hid_key_redskull,    180.2, 11.1, 0.1, 0.8);
   if(p->keys.ys) DrawSpriteAlpha(":HUD:H_KS2", hid_key_yellowskull, 170.2, 11.1, 0.1, 0.8);
   if(p->keys.bs) DrawSpriteAlpha(":HUD:H_KS3", hid_key_blueskull,   160.2, 10.1, 0.1, 0.8);
   if(p->keys.rc) DrawSpriteAlpha(":HUD:H_KC1", hid_key_red,         150.2, 10.1, 0.1, 0.8);
   if(p->keys.yc) DrawSpriteAlpha(":HUD:H_KC2", hid_key_yellow,      140.2, 11.1, 0.1, 0.8);
   if(p->keys.bc) DrawSpriteAlpha(":HUD:H_KC3", hid_key_blue,        130.2, 12.1, 0.1, 0.8);
}

// Extern Functions ----------------------------------------------------------|

//
// Activate
//
void Upgr_HeadsUpDis2_Activate(struct player *p, upgrade_t *upgr)
{
   p->hudenabled = true;
}

//
// Disable
//
void Upgr_HeadsUpDis2_Deactivate(struct player *p, upgrade_t *upgr)
{
   p->hudenabled = false;
}

//
// Render
//
void Upgr_HeadsUpDis2_Render(struct player *p, upgrade_t *upgr)
{
   // Log
   Lith_HUD_Log(p);

   // Inventory
   HUD_KeyInd(p);

   if(p->getCVarI("lith_hud_showscore"))
      HUD_Score(p);
   if(p->getCVarI("lith_hud_showweapons"))
      HUD_Weapons(p);

   // Status
   HUD_Ammo(p);
   HUD_Health(p);
}

// EOF

